"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.functionCommaSpaceChecker = functionCommaSpaceChecker;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var _postcssValueParser = require("postcss-value-parser");

var _postcssValueParser2 = _interopRequireDefault(_postcssValueParser);

var _utils = require("../../utils");

var ruleName = "function-comma-space-after";

exports.ruleName = ruleName;
var messages = (0, _utils.ruleMessages)(ruleName, {
  expectedAfter: function expectedAfter() {
    return "Expected single space after \",\"";
  },
  rejectedAfter: function rejectedAfter() {
    return "Unexpected whitespace after \",\"";
  },
  expectedAfterSingleLine: function expectedAfterSingleLine() {
    return "Expected single space after \",\" in a single-line list";
  },
  rejectedAfterSingleLine: function rejectedAfterSingleLine() {
    return "Unexpected whitespace after \",\" in a single-line list";
  }
});

exports.messages = messages;

exports["default"] = function (expectation) {
  var checker = (0, _utils.whitespaceChecker)("space", expectation, messages);
  return function (root, result) {
    var validOptions = (0, _utils.validateOptions)(result, ruleName, {
      actual: expectation,
      possible: ["always", "never", "always-single-line", "never-single-line"]
    });
    if (!validOptions) {
      return;
    }

    functionCommaSpaceChecker({
      root: root,
      result: result,
      locationChecker: checker.after,
      checkedRuleName: ruleName
    });
  };
};

function functionCommaSpaceChecker(_ref) {
  var locationChecker = _ref.locationChecker;
  var root = _ref.root;
  var result = _ref.result;
  var checkedRuleName = _ref.checkedRuleName;

  root.walkDecls(function (decl) {
    var functionOffset = (0, _utils.declarationValueIndexOffset)(decl);

    var cssFunctions = (0, _postcssValueParser2["default"])(decl.value).nodes.filter(function (node) {
      return node.type === "function";
    });

    cssFunctions.forEach(function (cssFunction) {
      var cssFunctionString = _postcssValueParser2["default"].stringify(cssFunction);
      (0, _utils.styleSearch)({ source: cssFunctionString, target: "," }, function (match) {
        locationChecker({
          source: cssFunctionString,
          index: match.startIndex,
          err: function err(message) {
            (0, _utils.report)({
              message: message,
              node: decl,
              index: functionOffset + cssFunction.sourceIndex + match.startIndex,
              result: result,
              ruleName: checkedRuleName
            });
          }
        });
      });
    });
  });
}